/* Analyze the data by calculating sums, sums of squares, and
   autocorrelations of all the measurable quantities, and
   binning the data to obtain errors.
   Current values are backed up and restored from the aves files.
*/

#include "comext.h"
#include <math.h>
#include <string.h>
#include <stdio.h>

analinit(first)
int first;
{
int i,j,m,n;
char lastavesname[128];
FILE *flast,*fopen();

  if (first) {  /* initialize everything */

    cntaccept=0; cntenergy=0; cntspin=0; 
    for (m=0;m<M;m++) { 
       avenergy[m]=0.; avenersq[m]=0.; avspheat[m]=0.; avspheatsq[m]=0.; 
       avspin[m]=0.; avspinsq[m]=0.; avsuss[m]=0.; avsusssq[m]=0.;
       avbinenersq[m]=0; avbinspinsq[m]=0;
       avmagsq[m]=0.;avbinmagsqsq[m]=0.;
       avimsuss[m]=0.; avimsusssq[m]=0.;
       minenergy[m] = 10.0; maxenergy[m] = -10.0;
       minspin[m] = 1.0; maxspin[m] = 0.0;
    }
    for (i=0;i<NA;i++) {
       for (m=0;m<M;m++) { 
          EA[m][i] = 0.0; EA2[m][i] = 0.0;
          MA[m][i] = 0.0; MA2[m][i] = 0.0;
          M2A[m][i] = 0.0; M2A2[m][i] = 0.0;
	  lastNEA[m][i] = 0.0;
	  lastNMA[m][i] = 0.0;
	  lastNM2A[m][i] = 0.0;
       }
    }
  }

  else {   /* Read results from previous aves file */

     sprintf(lastavesname, "aves.%s", lastfname);
     if ((flast = fopen(lastavesname, "r")) == NULL) {
        perror("lastaves open") ;
        exit( -1 ) ;
     }
     readaves(flast);
     fclose(flast);
  }

  for (m=0;m<M;m++) { 
     partenergy[m]=0.; partenersq[m]=0.; partspin[m]=0.;
     partspinsq[m]=0.; partmagsq[m]=0.;
  }
}


analdata()
{
int i,j,m,n;
int binnum;

   /* Energy */ 
   cntenergy++;
   analyze_energy();
 
   /* Magnetization (spin in the preferred direction) */
   cntspin++; 
   analyze_spin();
 
   /* Magnetization squared */
   analyze_magsq();

   if (cntenergy % binsz == 0) {  
      if (BINARY) {
	 binnum = (cntenergy - oldcount)/binsz;
	 printf("Bin number %d out of %d\n", binnum,numbins);
	 fflush(stdout);
      }
   }
}


/* FUNCTIONS */

analyze_energy()
{
int i,j,m,n;

   for (m=0;m<M;m++) {
      if (energy[m] < minenergy[m]) minenergy[m] = energy[m];
      if (energy[m] > maxenergy[m]) maxenergy[m] = energy[m];
      partenergy[m] += energy[m];
      partenersq[m] += energy[m] * energy[m];
      if ((cntenergy % binsz) == 0) {
	 binmeansq = (partenergy[m]*partenergy[m])/(binsz*binsz);
	 spheat[m] = partenersq[m]/binsz - binmeansq;
	 avspheat[m] += spheat[m];
	 avspheatsq[m] += spheat[m] * spheat[m];
         avenergy[m] += partenergy[m];
         avenersq[m] += partenersq[m];
	 avbinenersq[m] += binmeansq;
         partenergy[m] = 0.;
         partenersq[m] = 0.;
      }
      /* Energy auto-correlations */
      /* autocorrs(lastNEA,EA,EA2,energy,m); */
      autocorrs(lastNEA,EA,energy,m);
   }
}


analyze_spin()
{
int i,j,m,n;

   for (m=0;m<M;m++) {
      if (mag[m] < minspin[m]) minspin[m] = mag[m];
      if (mag[m] > maxspin[m]) maxspin[m] = mag[m];
      partspin[m] += mag[m];
      partspinsq[m] += mag[m] * mag[m];
      if (cntspin % binsz == 0) {
	 binmeansq = (partspin[m]*partspin[m])/(binsz*binsz);
	 suss[m] = partspinsq[m]/binsz - binmeansq;
	 avsuss[m] += suss[m];
	 avsusssq[m] += suss[m] * suss[m];
         avspin[m] += partspin[m];
         avspinsq[m] += partspinsq[m];
	 avbinspinsq[m] += binmeansq;
         partspin[m] = 0.;
         partspinsq[m] = 0.;
	 spinmeansq[m] = binmeansq;
      }
      /* Spin auto-correlations */
      /* autocorrs(lastNMA,MA,MA2,mag,m); */
      autocorrs(lastNMA,MA,mag,m);
   }
}


analyze_magsq()
{
int i,j,m,n;

   n = 0;
   partmagsq[n] += magsq[n];
   if (cntspin % binsz == 0) {
      avmagsq[n] += partmagsq[n];
      avbinmagsqsq[n] += (partmagsq[n] * partmagsq[n]) / (binsz*binsz);
      for (m=0;m<M;m++) {
         imsuss[m] = (partmagsq[n]/binsz) - spinmeansq[m];
         avimsuss[m] += imsuss[m];
         avimsusssq[m] += imsuss[m] * imsuss[m];
      }
      partmagsq[n] = 0.;
   }
   /* Spin squared auto-correlations */
   /* autocorrs(lastNM2A,M2A,M2A2,magsq,n); */
   autocorrs(lastNM2A,M2A,magsq,n);
}


autocorrs(lastNA,A,value,m)
double lastNA[M][MAXNA], A[M][MAXNA];
float value[M];
int m;
{
int i,j;

   for (i=1;i<NA;i++) lastNA[m][i-1] = lastNA[m][i];
   lastNA[m][NA-1] = value[m];
   for (i=0;i<NA;i++) {
      A[m][i] += lastNA[m][0] * lastNA[m][i];
   }
}

